#!/usr/bin/python3
"""
Convert an html file into a koi file.
"""

if __name__ == '__main__':

    import config
    import time
    import json
    import os
    import shutil
    import getpass
    import logging
    logging.disable(logging.INFO)
    from koi import __version__

    print('\nHTML to koi file converter')
    print('==========================\n')
    html = input('HTML file: ')
    with open(html, 'r', encoding='utf-8', errors='ignore') as fd:
        html = fd.read()
    koi_file = input('koi file (if updating): ')
    user = input('Your koi user name: ')
    user = user.lower()
    if koi_file:
        with open(koi_file, 'r', encoding='utf-8') as fd:
            koi_data = json.load(fd)
        koi_data["body"] = html
        now = int(time.time())
        koi_data["timestamp"] = now
        koi_data["koi_version"] = __version__
        koi_data["last_edit_by"] = user
        kbak = '.'+os.path.basename(koi_file)+'-'+koi_data["rev"]
        koi_data["rev"] = str(int(koi_data["rev"])+1).zfill(5)
        kpath = os.path.dirname(koi_file)
        shutil.copyfile(koi_file, os.path.join(kpath, kbak))
        with open(koi_file, 'w', encoding='utf-8') as fd:
            json.dump(koi_data, fd, ensure_ascii=False)
        os.chmod(koi_file, config.wwwfperms)
        if getpass.getuser() == 'root':
            shutil.chown(koi_file, config.wwwuid, config.wwwgid)
        print(f'\nUpdated koi file\n')
        raise SystemExit
    else:
        koi_file = "article.koi"
    print('\nAll subsequent fields are optional, press <enter> to skip\n')
    title = input('Title: ')
    author = input('Author: ')
    keywords = input('Keywords: ')
    markdown = input('Use Markdown [y/N]? ')
    if markdown.lower() == 'y':
        markdown = True
    else:
        markdown = False
    print('\nNote that, if untrusted, the article editor will destroy most markup')
    trusted = input('Trusted (arbitrary HTML allowed) [Y/n]? ')
    if trusted.lower() == 'n':
        trusted = False
    else:
        trusted = True
    now = int(time.time())
    acl = {koi_file: {"users": [user], "groups": [], "ips": "*",
                           "time": now}}
    koi_data = {"title": title, "keywords": keywords, "body": html, \
                "timestamp": now, "author": author, "koi_version": __version__, \
                "editors": [user], "editor_groups": [], "acl": acl, \
                "rev": "00000", "created": now, "trusted": trusted, \
                "markdown": markdown, 'notes': [], 'last_edit_by': user}
    with open(koi_file, 'w', encoding='utf-8') as fd:
        json.dump(koi_data, fd, ensure_ascii=False)
    os.chmod(koi_file, config.wwwfperms)
    if getpass.getuser() == 'root':
        shutil.chown(koi_file, config.wwwuid, config.wwwgid)
    print(f'\nCreated "article.koi", you can now move it into the appropriate web page directory')
    print('(remember to set the proper ownership/file permissions if need be). Run "koiacl.py"')
    print('to adjust the ACL.\n')
