#!/usr/bin/python3

EDITOR="emacs -nw"

if __name__ == '__main__':

    import config
    import time
    import json
    import os
    import tempfile
    import getpass
    import shutil
    import sys
    import logging
    logging.disable(logging.INFO)
    from koi import __version__

    print('\nkoi file editor')
    print('===============\n')
    if len(sys.argv) > 1:
        koi_file = sys.argv[1]
    else:
        koi_file = input('koi file: ')
    user = input('Your koi user name: ')
    user = user.lower()
    if os.path.isfile(koi_file):
        new = False
        with open(koi_file, 'r', encoding='utf-8') as fd:
            koi_data = json.load(fd)
    elif os.path.basename(koi_file) == 'article.koi':
        new = True
        print('Creating a new article, run "acledit.py" afterwards to change the ACL')
        koi_data = {}
        now = int(time.time())
        koi_data['rev'] = '0'.zfill(5)
        koi_data['created'] = now
        koi_data['trusted'] = False
        koi_data['markdown'] = True
        koi_data['notes'] = []
        koi_data['author'] = user
        koi_data['title'] = 'untitled'
        koi_data['keywords'] = ''
        koi_data['body'] = ''
        koi_data['editors'] = [user]
        koi_data['koi_version'] = __version__
        koi_data['last_edit_by'] = user
        koi_data['editor_groups'] = []
        koi_data['tags'] = []
        koi_data['acl'] = {'article.koi': {'users': [user], 'groups': [], \
                                           'ips': '*', 'time': now}}
    else:
        print('\nFile not found/not an "article.koi" file.\n')
        raise SystemExit
    editable = [key for key in koi_data if isinstance(koi_data[key], str)]
    editable.sort()
    print(f'Text fields found: {" ".join(editable)}')
    if 'body' in editable:
        default = 'body'
    else:
        default = editable[0]
    field = input(f'koi text field to edit [{default}]: ')
    if not field:
        field = default
    (fd, tmpfile) = tempfile.mkstemp()
    with os.fdopen(fd, 'w', encoding='utf8') as tmp:
        tmp.write(koi_data[field])
    while True:
        os.system(f'{EDITOR} {tmpfile}')
        with open(tmpfile, "r") as fd:
            koi_data[field] = fd.read()
        go = 'n'
        if os.path.splitext(os.path.basename(koi_file))[0] == "article":
            if not new:
                go = input('Update article metadata and create a revision backup? [Y/n]: ')
        if not go or go.lower() in ['y', 'yes']:
            print('Updating metadata')
            now = int(time.time())
            koi_data["timestamp"] = now
            koi_data["koi_version"] = __version__
            koi_data["last_edit_by"] = user
            kbak = '.'+os.path.basename(koi_file)+'-'+koi_data["rev"]
            koi_data["rev"] = str(int(koi_data["rev"])+1).zfill(5)
            kpath = os.path.dirname(koi_file)
            shutil.copyfile(koi_file, os.path.join(kpath, kbak))
        with open(koi_file, 'w', encoding='utf-8') as fd:
            json.dump(koi_data, fd, ensure_ascii=False)
        os.chmod(koi_file, config.wwwfperms)
        if getpass.getuser() == 'root':
            shutil.chown(koi_file, config.wwwuid, config.wwwgid)
        print(f'\nUpdated koi file: {os.path.realpath(koi_file)}\n')
        go = input('Continue editing? [Y/n]: ')
        if go.lower() in ['n', 'no']:
            break
    os.remove(tmpfile)
