#!/usr/bin/python3
"""
Koi portfolio generator. Creates either a "gallery.koi" or
"showcase.koi" file which can be used to create an online
portfolio.
"""

if __name__ == '__main__':

    import config
    import time
    import json
    import os
    import getpass
    import shutil
    import logging
    logging.disable(logging.INFO)
    from koi import __version__

    print('\nkoi Portfolio Generator')
    print('=======================\n')
    user = input('koi user name (curator): ')
    user = user.lower()
    gtype = input('Specify portfolio type [1=gallery (def), 2=showcase]: ')
    if not gtype:
        gtype = '1'
    print('\nAll subsequent fields are optional, press <enter> to skip\n')
    title = input('Title: ')
    public = input(f'Make public (if not ACL will only allow access to "{user}") [Y/n]? ')
    if public.lower() == 'n':
        public = False
    else:
        public = True
    hide = input('Hide new images (otherwise will need ?tag in the URL to view) [y/N]? ')
    if hide.lower() == 'y':
        hide = True
        tag = input('Unhide tag: ')
    else:
        hide = False
        tag = ''
    now = int(time.time())
    if gtype == '1':
        koi_file = 'gallery.koi'
        creator = input('Creator: ')
        keywords = input('Keywords: ')
        showcase = input('Showcase (if any, full path to dir): ')
        koi_data = {'title': title, 'keywords': keywords, 'notes': [], 'tags': [], \
                    'timestamp': now, 'creator': creator, 'koi_version': __version__, \
                    'editors': [user], 'editor_groups': [], "showcase": showcase}
        koi_data['thumbnail'] = {'size': [260, 260], 'quality': 90, 'optimize': True, \
                                 'type': 'JPEG', 'square': True, 'frame': True, \
                                 'progressive': True}
        koi_data['histogram'] = {'height': 150, 'fg_col': [80, 80, 80], \
                                 'bg_col': [180, 180, 180], 'lines_col': [220, 220, 220], \
                                 'border_col': [60, 60, 60], 'show': True, 'n_lines': 5, \
                                 'create': True}
        koi_data['get_users'] = True
        koi_data['created'] = now
    else:
        koi_file = 'showcase.koi'
        obf = input('Obfuscate image names [y/N]? ')
        if obf.lower() == 'y':
            obf = True
        else:
            obf = False
        koi_data = {'title': title, 'obfuscate': obf}
    if public:
        koi_data['acl'] = {f'{koi_file}': {'users': '*', 'groups': [], \
                                    'ips': '*', 'time': now}}
    else:
        koi_data['acl'] = {f'{koi_file}': {'users': [user], 'groups': [], \
                                    'ips': '*', 'time': now}}
    koi_data['image'] = {'formats': ['.jpg', '.jpeg', '.png'], 'unhide_tag': tag, \
                         'hide_new': hide}
    koi_data['library'] = {}
    with open(koi_file, 'w', encoding='utf-8') as fd:
        json.dump(koi_data, fd, ensure_ascii=False)
    os.chmod(koi_file, config.wwwfperms)
    if getpass.getuser() == 'root':
        shutil.chown(koi_file, config.wwwuid, config.wwwgid)
    print(f'\nCreated "{koi_file}", you can now move it into the appropriate web page directory')
    print('(remember to set the proper ownership/file permissions if need be). Run "koiacl.py"')
    print('to adjust the ACL.\n')
